DECLARE SUB Measurement ()
DECLARE SUB DisplayGraph ()
DECLARE SUB WriteResults ()
DECLARE SUB CloseComm ()
DECLARE SUB WriteTHS (s$)
DECLARE FUNCTION ReaDThs$ ()
DECLARE SUB OpenComm ()
DECLARE SUB GetCommandLineOptions ()
DEFINT A-Z
DIM SHARED StartDate$, StopDate$, StartTime$, StopTime$
DIM SHARED Count AS LONG, NumOfSamples AS LONG, SampleInterval AS LONG
DIM SHARED Verbose, CommPort, THS, LogF, KeyPressed, ErrNum
DIM SHARED ID$, DMM$
DIM SHARED Logfile AS STRING * 12
DIM SHARED Min AS SINGLE, Mean AS SINGLE, Max AS SINGLE
DIM SHARED MakeGraph, MakeGraphOnly
DIM SHARED Version$

CONST Ver$ = "0.96"
CONST tmpfile1$ = "dmm1.tmp"
CONST tmpfile2$ = "dmm2.tmp"

'-------------------------------------------------------------------------
Version$ = Ver$
CALL GetCommandLineOptions                          'process commandline options

ON ERROR GOTO ErrorHandler                          'if error, test it
ON TIMER(SampleInterval) GOSUB DoAcq

IF MakeGraphOnly THEN                               'if it is only graph, then do it
    DisplayGraph
    END                                             'exit program
END IF

KILL Logfile$                                       'kill the file if it exists
CALL OpenComm                                       'make communication
CALL Measurement                                    'make the measurements
CALL WriteResults                                   'save the results
CALL CloseComm                                      'close communication

IF MakeGraph THEN
    DisplayGraph                                    'do we want to graph ?
END IF

END

ErrorHandler:
    ErrNum = ERR
    RESUME NEXT

DoAcq:
    WriteTHS "DMM:VAL?"
    R$ = ReaDThs$: M! = VAL(R$)
    IF M! < Min THEN Min = M!                       'see if MIN
    IF M! > Max THEN Max = M!                       'see if max
    Mean = Mean + M!                                'calc mean

    Meas$ = LTRIM$(STR$(M!))
    PRINT #LogF, USING "####.####"; M!              'save to file
    LOCATE , 1
    Count = Count + 1
    PRINT Count, Meas$, TAB(30);                    'display values
 
    IF Count >= NumOfSamples THEN                   'end of loop ?
        KeyPressed = 1                              'indicate
    END IF
   
    RETURN

SUB CloseComm

CLOSE THS                                           'shutdown communication
END SUB

SUB DisplayGraph

F = FREEFILE                                        'free file
OPEN Logfile$ FOR INPUT AS F                        'open data file
INPUT #F, Version$, ID$                             'read various variables
INPUT #F, StartDate$, StartTime$, StopDate$, StopTime$
INPUT #F, NumOfSamples, SampleInterval
INPUT #F, Min, Mean, Max
LINE INPUT #F, DMM$                                 'read dmm settings
INPUT #F, t$, t$, t$

SCREEN 12                                          'VGA screen (640x480)
VIEW (1, 1)-(540, 380), , 7                        'define area
WINDOW (1, Min)-(NumOfSamples, Max)                'define window

INPUT #F, M!                                       'read first point
PSET (1, M!)                                       'set first point

FOR tt& = 2 TO NumOfSamples                        'read all data points
    INPUT #F, M!
    LINE -(tt&, M!)                                'draw line
NEXT tt&                                           'make loop

CLOSE #F                                           'close file

LINE (1, Mean)-(NumOfSamples, Mean), , , 240       'draw mean line

LOCATE 1, 69: PRINT Max;                           'annotate graph
LOCATE 24, 69: PRINT Min;
LOCATE 25, 1: PRINT "1";
LOCATE 25, 30: PRINT "seconds";
LOCATE 25, 64: PRINT USING "#####"; NumOfSamples * SampleInterval;
LOCATE 26, 1:
PRINT "Start date : " + StartDate$; TAB(27); "Min   :" + STR$(Min); TAB(50); "# of samples    :"; STR$(NumOfSamples);
PRINT "Start time : " + StartTime$; TAB(27); "Mean  :" + STR$(Mean); TAB(50); "Sample interval :"; STR$(SampleInterval); " sec";
PRINT "Stop date  : " + StopDate$; TAB(27); "Max   :" + STR$(Max)
PRINT "Stop time  : " + StopTime$; TAB(27); "Instr : " + ID$;

DO
LOOP WHILE INKEY$ = ""                              'wait until key pressed

SCREEN 0                                            'reset screen

END SUB

SUB GetCommandLineOptions

C$ = COMMAND$
IF INSTR(C$, "-?") OR INSTR(C$, "-H") THEN
    PRINT "dmmlog v" + Version$
    PRINT "usage: dmmlog [options]"
    PRINT "  options:"
    PRINT "    -? -H        help"
    PRINT "    -Cx          commport                 default 1"
    PRINT "    -Nx          number of samples        default 100"
    PRINT "    -Ix          sample interval          default 10"
    PRINT "    -Fname       file name for results    default dmmlog.txt"
    PRINT "    -V           verbose"
    PRINT "    -G           graph after logging"
    PRINT "    -O           graph only, no logging"
    END
END IF

i = INSTR(C$, "-C")                                 'proces the commport number 
IF i THEN
    CommPort = VAL(MID$(C$, i + 2, 1))
ELSE
    CommPort = 1
END IF

i = INSTR(C$, "-N")                                 'proces the number of samples    
IF i THEN
    NumOfSamples = VAL(MID$(C$, i + 2, 6))
ELSE
    NumOfSamples = 100
END IF

i = INSTR(C$, "-I")                                 'process the sample interval 
IF i THEN
    SampleInterval = VAL(MID$(C$, i + 2, 6))
ELSE
    SampleInterval = 10
END IF

i = INSTR(C$, "-F")                                 'process the filename  
IF i THEN
    s = INSTR(i, C$, " ")                           'find space
    IF s = 0 THEN s = 12                            'probably at end of line
    Logfile$ = MID$(C$, i + 2, s - i + 2)
ELSE
    Logfile$ = "DMMLOG.TXT"
END IF

IF INSTR(C$, "-V") THEN Verbose = 1                 'verbose option

IF INSTR(C$, "-G") THEN MakeGraph = 1               'need to graph ?

IF INSTR(C$, "-O") THEN MakeGraphOnly = 1           'graph only ?

END SUB

SUB Measurement

StartTime$ = TIME$: StartDate$ = DATE$              'save starting time&date
KILL tmpfile1$                                      'kill temp file (if it is there)
LogF = FREEFILE: OPEN tmpfile1$ FOR OUTPUT AS LogF  'open temp file
IF Verbose THEN PRINT "Logging" + STR$(NumOfSamples) + " measurements with" + STR$(SampleInterval) + " seconds interval ..."
PRINT "Press any key to terminate logging ..."      'stop program
Max = -9.9E+11: Min = 9.9E+11                       'define values for min/max

TIMER ON                                            'start measurement
DO
    IF INKEY$ <> "" THEN
        KeyPressed = 1
        PRINT : PRINT "Logging terminated"
    END IF
LOOP UNTIL KeyPressed
TIMER OFF

PRINT : CLOSE LogF                                        'close file
StopTime$ = TIME$: StopDate$ = DATE$                'get end time&date
IF Verbose THEN PRINT "Logging done ..."

END SUB

SUB OpenComm
THS = FREEFILE
SELECT CASE CommPort
        CASE 1: OPEN "COM1:9600,N,8,1,LF,OP2500" FOR RANDOM AS THS
        CASE 2: OPEN "COM2:9600,N,8,1,LF,OP2500" FOR RANDOM AS THS
END SELECT
IF ErrNum = 24 THEN
    PRINT "Cannot establish communication, check configuration."
    PRINT "THS7XX settings : RS232 default, then change EOL to CR."
    PRINT "Currently using PC comm port" + STR$(CommPort) + "."
    END
END IF

IF Verbose THEN PRINT "Establishing communication using commport" + STR$(CommPort) + " ..."

WriteTHS "*IDN?": ID$ = ReaDThs$                    'ask ID --> IEEE488.2 !!!
IF Verbose THEN PRINT "Instrument identified as " + ID$

IF INSTR(ID$, "THS 7") = 0 THEN
    PRINT "No TEKTRONIX THS 7xx found, quitting program.."
    CALL CloseComm
    END
END IF

WriteTHS ":VERBOSE 0;:HEADER 1;:DMM?"               'ask dmm settings
DMM$ = ReaDThs$                                     'read settings
WriteTHS ":HEADER 0"                                'no header
IF Verbose THEN PRINT "THS DMM settings : " + DMM$

END SUB

FUNCTION ReaDThs$

LINE INPUT #THS, t$
ReaDThs$ = t$
END FUNCTION

SUB WriteResults
IF Verbose THEN PRINT "Gathering additional information ..."
KILL tmpfile2$: LogF = FREEFILE: OPEN tmpfile2$ FOR OUTPUT AS LogF
WRITE #LogF, Version$, ID$
WRITE #LogF, StartDate$, StartTime$, StopDate$, StopTime$
WRITE #LogF, Count, SampleInterval
PRINT #LogF, USING "####.####"; Min; : PRINT #LogF, ",";
PRINT #LogF, USING "####.####"; Mean / Count; : PRINT #LogF, ",";
PRINT #LogF, USING "####.####"; Max
WRITE #LogF, DMM$
PRINT #LogF, "Reserved1"
PRINT #LogF, "Reserved2"
PRINT #LogF, "Reserved3"
CLOSE LogF
IF Verbose THEN PRINT "Combining files, saving result in " + Logfile$
SHELL "copy " + tmpfile2$ + "+" + tmpfile1$ + " " + Logfile$ + "/b >nul"
IF Verbose THEN PRINT "Deleting temporary files ..."
KILL tmpfile1$
KILL tmpfile2$

END SUB

SUB WriteTHS (s$)
PRINT #THS, s$
END SUB

